import { reactive, toRaw } from 'vue'

// 分页钩子函数
interface Options {
    page?: number
    size?: number
    fetchFun: (_arg: any) => Promise<any>
    params?: Record<any, any>
    firstLoading?: boolean
}

function guolvParams(params: any) {
    let p: any = {};
    for (let key in params) {
        if (params[key] && params[key] != 'all' || params[key] === 0) {
            p[key] = params[key];
        }
    }
    return p
}

export function usePaging(options: Options) {
    let { page = 1, size = 15, fetchFun, params = {}, firstLoading = false } = options

    // 记录分页初始参数
    const paramsInit: Record<any, any> = Object.assign({}, toRaw(params))
    // 分页数据
    const pager = reactive({
        page,
        size,
        loading: firstLoading,
        count: 0,
        lists: [] as any[],
        extend: {} as Record<string, any>
    })

    if (params.rows) {
        pager.size = params.rows
    }
    // 请求分页接口
    const getLists = (_params = {}) => {
        pager.loading = true;

        if (JSON.stringify(_params) != '{}') {
            params = { ...params, ..._params }
        }

        // console.log(params)

        return fetchFun({
            page: pager.page,
            rows: pager.size,
            ...guolvParams(params)
        })
            .then((res: any) => {

                if (params.proxy) {
                    pager.count = res.data.rowCount || res.data.totalRow;
                    pager.lists = res?.data.list;
                } else {
                    if (res?.data.results) {
                        pager.count = res?.data.results.totalRow;
                        pager.lists = res?.data.results.list;
                    } else {
                        pager.lists = res?.data.list;

                        if (res?.data.totalRow) {
                            pager.count = Number(res?.data.totalRow);
                        } else {
                            pager.count = res?.data.list.length;
                        }
                    }
                    pager.extend = res?.data;
                    return Promise.resolve(res?.data)
                }
            })
            .catch((err: any) => {
                return Promise.reject(err)
            })
            .finally(() => {
                pager.loading = false
            })
    }
    // 重置为第一页
    const resetPage = () => {
        pager.page = 1
        getLists()
    }

    // 重置参数
    const resetParams = () => {
        Object.keys(paramsInit).forEach((item) => {
            params[item] = paramsInit[item]
        })
        getLists()
    }
    return {
        pager,
        getLists,
        resetParams,
        resetPage
    }
}
