import { defineStore } from 'pinia'
import cache from '@/utils/cache'
import type { RouteRecordRaw } from 'vue-router'
import { getUserInfo, login } from '@/api/user'
import router, { filterAsyncRoutes } from '@/router'
import { TOKEN_KEY } from '@/enums/cacheEnums'
import { PageEnum } from '@/enums/pageEnum'
import { clearAuthInfo, getToken } from '@/utils/auth'
import menu from '@/router/menu'
import { shopType1, shopType2, shopType3, shopType4 } from "./auth"

export interface UserState {
    token: string
    userInfo: Record<string, any>
    routes: RouteRecordRaw[]
    perms: string[]
}

const useUserStore = defineStore({
    id: 'user',
    state: (): UserState => ({
        token: getToken() || '',
        // 用户信息
        userInfo: {},
        // 路由
        routes: [],
        // 权限
        perms: []
    }),
    getters: {},
    actions: {

        resetState() {
            this.token = ''
            this.userInfo = {}
            this.perms = []
        },

        login(playload: any) {
            const { name, password } = playload
            return new Promise((resolve, reject) => {

                login({
                    name: name.trim(),
                    password: password
                })
                    .then((res) => {
                        this.token = res.data.token
                        cache.set(TOKEN_KEY, res.data.token)
                        resolve(res.data)
                    })
                    .catch((error) => {
                        reject(error)
                    })

            })
        },

        async logout() {
            this.token = ''
            await router.push(PageEnum.LOGIN)
            clearAuthInfo()
        },

        getUserInfo() {
            return new Promise((resolve, reject) => {
                getUserInfo()
                    .then((res: any) => {

                        this.userInfo = res.data;
                        let perms = ['*'];

                        this.perms = perms;
                        this.routes = filterAsyncRoutes(menu, true, this.perms);

                        resolve({
                            ...res.data,
                            permissions: [],
                            menu
                        })

                    })
                    .catch((error) => {
                        reject(error)
                    })
            })
        }
    }
})

export default useUserStore
